package util

import (
	"math/big"

	"github.com/ethereum/go-ethereum/accounts/abi"
	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/common/math"
	"github.com/ethereum/go-ethereum/crypto"
	"github.com/ethereum/go-ethereum/signer/core/apitypes"
)

func PermitSign(k []byte, name, version string, chainId, nextNonce *big.Int, contract, owner, spender common.Address) (v uint8, r, s [32]byte, err error) {
	typedData := apitypes.TypedData{
		Types: apitypes.Types{
			"EIP712Domain": []apitypes.Type{
				{Name: "name", Type: "string"},
				{Name: "version", Type: "string"},
				{Name: "chainId", Type: "uint256"},
				{Name: "verifyingContract", Type: "address"},
			},
			"Permit": []apitypes.Type{
				{Name: "owner", Type: "address"},
				{Name: "spender", Type: "address"},
				{Name: "value", Type: "uint256"},
				{Name: "nonce", Type: "uint256"},
				{Name: "deadline", Type: "uint256"},
			},
		},
		PrimaryType: "Permit",
		Domain: apitypes.TypedDataDomain{
			Name:              name, // 代币名称
			Version:           version,
			ChainId:           math.NewHexOrDecimal256(chainId.Int64()),
			VerifyingContract: contract.Hex(),
		},
		Message: apitypes.TypedDataMessage{
			"owner":    owner.Hex(),
			"spender":  spender.Hex(),
			"value":    abi.MaxUint256,
			"nonce":    nextNonce,
			"deadline": big.NewInt(2000000000),
		},
	}
	domainSeparator, err := typedData.HashStruct("EIP712Domain", typedData.Domain.Map())
	if err != nil {
		return
	}

	typedDataHash, err := typedData.HashStruct(typedData.PrimaryType, typedData.Message)

	sig, err := crypto.Sign(crypto.Keccak256([]byte("\x19\x01"), domainSeparator, typedDataHash), crypto.ToECDSAUnsafe(k))
	if err != nil {
		return
	}

	v = sig[64] + 27
	copy(r[:], sig[:32])
	copy(s[:], sig[32:64])
	return
}
