package util

import (
	"encoding/hex"
	"fmt"
	"math/big"
	"testing"

	"github.com/ethereum/go-ethereum/accounts/abi"
	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/common/math"
	"github.com/ethereum/go-ethereum/crypto"
	"github.com/ethereum/go-ethereum/signer/core/apitypes"
)

func Test2(t *testing.T) {
	v, r, s, e := PermitSign(common.Hex2Bytes("9acd67dc1d0382852501f9970aff6c3641b947c03d618a8e145396dbb62aac35"),
		"AON Application Score",
		"1",
		big.NewInt(84532),
		big.NewInt(0),
		common.HexToAddress("0x88F3EE47113B64caF99e47af4f51333d55d86d85"),
		common.HexToAddress("0xbDE4bD1AE8c0E49940904E721b68a278f4eDBfaf"),
		common.HexToAddress("0x88F3EE47113B64caF99e47af4f51333d55d86d85"),
	)
	t.Log(e)
	t.Log(v)
	t.Log(common.Bytes2Hex(r[:]))
	t.Log(common.Bytes2Hex(s[:]))
}

func TestPermit(t *testing.T) {
	chainId := big.NewInt(1)
	ca := common.HexToAddress("0xec53bF9167f50cDEB3Ae105f56099aaaB9061F83")
	typedata := apitypes.TypedData{
		Types: apitypes.Types{
			"EIP712Domain": []apitypes.Type{
				{Name: "name", Type: "string"},
				{Name: "version", Type: "string"},
				{Name: "chainId", Type: "uint256"},
				{Name: "verifyingContract", Type: "address"},
			},
			"Permit": []apitypes.Type{
				{Name: "owner", Type: "address"},
				{Name: "spender", Type: "address"},
				{Name: "value", Type: "uint256"},
				{Name: "nonce", Type: "uint256"},
				{Name: "deadline", Type: "uint256"},
			},
		},
		PrimaryType: "Permit",
		Domain: apitypes.TypedDataDomain{
			Name:              "EIGEN", // 代币名称
			Version:           "1",
			ChainId:           math.NewHexOrDecimal256(chainId.Int64()), // 链 ID
			VerifyingContract: ca.Hex(),                                 // 代币合约地址
		},
		Message: apitypes.TypedDataMessage{
			"owner":    common.HexToAddress("0x00ad3D4cC099Be937879451698C673CBD5C3CEfc").String(),
			"spender":  common.HexToAddress("0x75d7B715DD175DE4b75A99c1ee2ae278b25BDBbA").String(),
			"value":    abi.MaxUint256,
			"nonce":    big.NewInt(0),
			"deadline": big.NewInt(2000000000),
		},
	}
	domainSeparator, err := typedata.HashStruct("EIP712Domain", typedata.Domain.Map())
	t.Log(err)
	t.Log(domainSeparator.String())

	typedDataHash, err := typedata.HashStruct(typedata.PrimaryType, typedata.Message)
	if err != nil {
		t.Log(err)
	}
	fmt.Println("domainSeparator:", hex.EncodeToString(domainSeparator))
	fmt.Println("typedDataHash:", hex.EncodeToString(typedDataHash))

	k := crypto.ToECDSAUnsafe(common.Hex2Bytes("9acd67dc1d0382852501f9970aff6c3641b947c03d618a8e145396dbb62aac35"))
	// b09b7edc6cf15e9790f017cb13819e21f8b9a6d32163e3c4ec97562506084ec3
	t.Log(common.Bytes2Hex(crypto.Keccak256([]byte("\x19\x01"), domainSeparator, typedDataHash)))
	sig, err := crypto.Sign(crypto.Keccak256([]byte("\x19\x01"), domainSeparator, typedDataHash), k)
	t.Log(common.Bytes2Hex(sig))
}
